@@ -3,7 +3,7 @@ use v5.24;
 use warnings;
 use experimental qw< signatures >;
 no warnings qw< experimental::signatures >;
-{ our $VERSION = '0.007002' }
+{ our $VERSION = '0.011' }
 
 use Exporter 'import';
 our @EXPORT_OK = qw< d run >;
@@ -168,9 +168,11 @@ sub d (@stuff) {
    warn Data::Dumper::Dumper(@stuff % 2 ? \@stuff : {@stuff});
 } ## end sub d (@stuff)
 
-sub default_getopt_config ($has_children) {
+sub default_getopt_config ($self, $spec) {
    my @r = qw< gnu_getopt >;
-   push @r, qw< require_order pass_through > if $has_children;
+   push @r, qw< require_order pass_through >
+      if has_children($self, $spec);
+   push @r, qw< pass_through > if  $spec->{'allow-residual-options'};
    return \@r;
 }
 
@@ -193,6 +195,21 @@ sub fetch_subcommand_default ($self, $spec) {
 }
 
 sub fetch_subcommand ($self, $spec, $args) {
+   my ($subc, $alias) = fetch_subcommand_wh($self, $spec, $args)
+      or return;
+   my $r = ref $subc;
+   if ($r eq 'HASH') {
+      $subc = $spec->{children}[$subc->{index}]
+         if scalar(keys $subc->%*) == 1 && defined $subc->{index};
+      $r = ref $subc;
+      return ($subc, $subc->{supports}[0]) if $r eq 'HASH';
+      $alias = $subc;
+   }
+   die "invalid sub-command (ref to $r)" if $r;
+   return ($subc, $alias);
+}
+
+sub fetch_subcommand_wh ($self, $spec, $args) {
    # if there's a dispatch, use that to figure out where to go next
    # **this** might even overcome having children at all!
    for my $cfg ($spec, $self->{application}{configuration}) {
@@ -234,7 +251,7 @@ sub fetch_subcommand ($self, $spec, $args) {
    shift @names;    # remove first one
    my $path = join '/', @names, $args->[0]; # $args->[0] was the candidate
    die "cannot find sub-command '$path'\n";
-} ## end sub fetch_subcommand
+} ## end sub fetch_subcommand_wh
 
 sub generate_factory ($c) {
    my $w = \&stock_factory;    # default factory
@@ -468,10 +485,15 @@ sub print_help ($self, $target) {
    }
 }
 
-sub stock_SpecFromHash ($s, $cmd) { $s->{application}{commands}{$cmd} }
+sub stock_SpecFromHash ($s, $cmd) {
+   return $cmd if ref($cmd) eq 'HASH';
+   return $s->{application}{commands}{$cmd} // undef;
+}
 
 sub stock_SpecFromHashOrModule ($s, $cmd) {
-   $s->{application}{commands}{$cmd} //= $s->{factory}->($cmd, 'spec')->();
+   return $cmd if ref($cmd) eq 'HASH';
+   return $s->{application}{commands}{$cmd}
+      //= $s->{factory}->($cmd, 'spec')->();
 }
 
 sub fetch_spec_for ($self, $command) {
@@ -525,8 +547,8 @@ sub sources ($self, $spec, $args) {
 
 sub stock_CmdLine ($self, $spec, $args) {
    my @args = $args->@*;
-   my $goc  = $spec->{getopt_config}
-     // default_getopt_config(scalar(get_children($self, $spec)));
+   my $goc  = $spec->{'getopt-config'}
+     // default_getopt_config($self, $spec);
    require Getopt::Long;
    Getopt::Long::Configure('default', $goc->@*);
 
@@ -542,7 +564,7 @@ sub stock_CmdLine ($self, $spec, $args) {
      or die "bailing out\n";
 
    # Check if we want to forbid the residual @args to start with a '-'
-   my $strict = !$spec->{'allow-residual-optionss'};
+   my $strict = !$spec->{'allow-residual-options'};
    if ($strict && @args && $args[0] =~ m{\A -}mxs) {
       Getopt::Long::Configure('default', 'gnu_getopt');
       Getopt::Long::GetOptionsFromArray(\@args, {});
